@@ -2,6 +2,7 @@ import asyncio
 from aiohttp import web
 from ..utils import Client
 import logging
+from inspect import isawaitable
 
 _logger = logging.getLogger(__name__)
 
@@ -27,53 +28,71 @@ class DebuggingClient(Client):
     def __init__(self, websocket, servers):
         super(DebuggingClient, self).__init__(websocket)
         self.servers = servers
+        self.auto_subscribe = False
 
     async def on_read(self, data):
-        if data.get('action') == 'list':
-            await self.send_queue.put({
-                "type": "instances",
-                "instances": [
-                    {
-                        "uuid": server.uuid,
-                        "tags": server.tags
-                    }
-                    for server in
-                    self.servers
-                ]
-            })
-        elif data.get('action') == 'subscribe':
-            for server in self.servers:
-                if server.uuid == data.get('uuid'):
-                    server.subscribe(self)
-        elif data.get('action') == 'unsubscribe':
-            for server in self.servers:
-                if server.uuid == data.get('uuid'):
-                    server.unsubscribe(self)
-        elif data.get('action') == 'call':
-            for server in self.servers:
-                if server.uuid == data.get('uuid'):
-                    await server.send_queue.put(data.get('params'))
-        elif data.get('action') == 'add_user':
-            username = data.get('username')
-            if username not in users:
-                users[username] = set()
-        elif data.get('action') == 'rm_user':
-            username = data.get('username')
-            if username in users:
-                del users[username]
-        elif data.get('action') == 'kill':
-            to_remove = set()
+        event_method = 'on_event_{}'.format(data.get('action'))
 
-            for server in self.servers:
-                if server.uuid == data.get('uuid'):
-                    to_remove.add(server)
+        if hasattr(self, event_method):
+            result = getattr(self, event_method)(data)
+            if isawaitable(result):
+                await result
+
+    async def on_event_call(self, data):
+        for server in self.servers:
+            if server.uuid == data.get('uuid'):
+                await server.send_queue.put(data.get('params'))
+
+    def on_event_subscribe(self, data):
+        for server in self.servers:
+            if server.uuid == data.get('uuid'):
+                server.subscribe(self)
+
+    def on_event_unsubscribe(self, data):
+        for server in self.servers:
+            if server.uuid == data.get('uuid'):
+                server.unsubscribe(self)
+
+    def on_event_add_user(self, data):
+        username = data.get('username')
+        if username not in users:
+            users[username] = set()
+
+    def on_event_rm_user(self, data):
+        username = data.get('username')
+        if username in users:
+            del users[username]
+
+    def on_event_autosubscribe(self, data):
+        self.auto_subscribe = data.get('params', [False])[0]
+
+    async def on_event_kill(self, data):
+        to_remove = set()
+
+        for server in self.servers:
+            if server.uuid == data.get('uuid'):
+                to_remove.add(server)
+
+        for server in to_remove:
+            self.servers.remove(server)
+            try:
+                await server.close()
+            except Exception:
+                pass
+
+    async def on_event_list(self, data):
+        await self.send_queue.put({
+            "type": "instances",
+            "instances": [
+                {
+                    "uuid": server.uuid,
+                    "tags": server.tags
+                }
+                for server in
+                self.servers
+            ]
+        })
 
-            for server in to_remove:
-                self.servers.remove(server)
-                try:
-                    await server.close()
-                except Exception:
-                    pass
 
 debugging_sessions = set()
 debugging_clients = set()
@@ -105,6 +124,11 @@ async def start_aiohttp():
             client.uuid = data.get('uuid')
             client.tags = data.get('tags', [])
             debugging_sessions.add(client)
+
+            for sb_client in debugging_clients:
+                if sb_client.auto_subscribe:
+                    client.subscribe(sb_client)
+
         elif data.get('event') == 'new_client':
             _logger.info("Got new client")
             data = await ws.receive_json()
