@@ -44,9 +44,15 @@ class Client(object):
                 self.message = await self.send_queue.get()
 
             message = self.message
+
             _logger.debug("Sending message [%s]", self.message)
-            await self.socket.send_json(self.message)
-            _logger.debug("Message sent")
+            try:
+                await self.socket.send_json(self.message)
+                _logger.debug("Message sent")
+            except TypeError:
+                msg = "Couldn't properly serialize to json : {}"
+                _logger.info(msg.format(self.message), exc_info=True)
+
             self.message = None
             return message
 
